import os
import os.path
import subprocess
from typing import List

from pydmt.configs import ConfigSubprocess


def check_call(args: List[str]) -> None:
    if ConfigSubprocess.print_command:
        print(f"running {args}")
    if ConfigSubprocess.quiet:
        subprocess.check_call(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    else:
        subprocess.check_call(args)


def check_call_ve_env(args: List[str]) -> None:
    virtual_env = os.getenv("VIRTUAL_ENV")
    assert virtual_env is not None, "not in virtual env"
    args[0] = os.path.join(virtual_env, "bin", args[0])
    if ConfigSubprocess.print_command:
        print(f"running {args}")
    if ConfigSubprocess.quiet:
        subprocess.check_call(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    else:
        subprocess.check_call(args)


def check_call_ve(orig_args: List[str]) -> None:
    # we call 'venv-run' with absolute path since it may change folder
    args = [
        "venv-run",
        "--venv",
        os.path.abspath(".venv/default"),
        "--",
    ]
    args.extend(orig_args)
    if ConfigSubprocess.print_command:
        print(f"running {args}")
    if ConfigSubprocess.quiet:
        subprocess.check_call(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    else:
        subprocess.check_call(args)



import os
import subprocess
import pydmt.helpers.project

deb_package = True
deb_section = 'python'
deb_priority = 'optional'
deb_architecture = 'all'
deb_package_name = pydmt.helpers.project.get_name()
# to which series to publish the package?
deb_series = [
    'artful',
    'zesty',
    'xenial',
    'trusty',
]
deb_depends = '${misc:Depends}, ${python3:Depends}, python3-mako'
deb_build_depends = 'python3, python3-setuptools, debhelper, dh-python'
deb_standards_version = '3.9.8'
deb_x_python_version = '>= 3.4'
deb_x_python3_version = '>= 3.4'
deb_urgency = 'low'
# archive of package
deb_repo = os.path.expanduser('~/packages')
# where to put packages that are built?
deb_out_folder = 'out'
# build_all = build.all
# where to build source packages?
# build_source = build.source
# where to build source packages?
# build_gbp = build.gbp

# for example 'Mon, 17 Oct 2016 09:44:00 +0300'
deb_date = subprocess.check_output("date -R").decode().rstrip()



import os

from pydmt.api.feature import Feature
from pydmt.builders.yaml import BuilderYaml
from pydmt.core.pydmt import PyDMT


class FeatureYaml(Feature):
    def __init__(
        self,
        yaml_folder: str = "yaml",
        validation_folder: str = "out"
    ):
        self.yaml_folder = yaml_folder
        self.validation_folder = validation_folder

    def setup(self, pydmt: PyDMT) -> None:
        if not os.path.isdir(self.yaml_folder):
            return
        for root, _, filenames in os.walk(self.yaml_folder):
            for filename in filenames:
                source = os.path.join(root, filename)
                target_base, ext = os.path.splitext(source)
                if ext == ".yaml":
                    target = os.sep.join([self.validation_folder, target_base, source + ".stamp"])
                    builder = BuilderYaml(
                        source=source,
                        target=target,
                    )
                    pydmt.add_builder(builder)



"""
This module builds python virtual envrionments
"""


import os
import shutil

from pydmt.utils.filesystem import mkdir_touch
from pydmt.utils.subprocess import check_call, check_call_ve
from pydmt.utils.python import collect_reqs, collect_bootstrap_reqs, get_install_args

from pydmt.api.one_source_one_target import OneSourceOneTarget
from pydmt.configs import ConfigVenv

SOURCE_FILE = "config/python.py"
TARGET_FOLDER = ".venv/default"


class BuilderVenv(OneSourceOneTarget):
    """
    This is a review of how to build a python virtual environment:
    # create the virtualenv
    virtualenv [folder]
    # activate it
    source [folder]/bin/activate
    # install package
    pip install -r requirements.txt
    """
    def build(self) -> None:
        # remove previous virtual env
        if os.path.isdir(TARGET_FOLDER):
            shutil.rmtree(TARGET_FOLDER)
        # create new virtual env
        args = [
            "virtualenv",
        ]
        if ConfigVenv.system_site_packages:
            args.append("--system-site-packages")
        args.append(TARGET_FOLDER)
        check_call(args)
        # install bootstrap packages so that we could read config/* files
        packs = collect_bootstrap_reqs()
        if packs:
            args = get_install_args()
            args.extend(packs)
            check_call_ve(args)
        # now install regular packages (we only run the install if there are packages to install)
        packs = collect_reqs()
        if packs:
            args.extend(packs)
            args = get_install_args()
            args.extend(packs)
            check_call_ve(args)
        mkdir_touch(self.target)



import os
import importlib

import pydmt.helpers.project


def get_github_username():
    mod = importlib.import_module("config.personal")
    if hasattr(mod, "github_username"):
        return getattr(mod, "github_username")
    return os.getlogin()


def get_launchpad_username():
    mod = importlib.import_module("config.personal")
    if hasattr(mod, "launchpad_username"):
        return getattr(mod, "launchpad_username")
    return os.getlogin()


def get_website():
    github_username = get_github_username()
    name = pydmt.helpers.project.get_name()
    return f"https://{github_username}.github.io/{name}"


def get_website_source():
    github_username = get_github_username()
    name = pydmt.helpers.project.get_name()
    return f"https://github.com/{github_username}/{name}"


def get_website_git():
    github_username = get_github_username()
    name = pydmt.helpers.project.get_name()
    return f"git://github.com/{github_username}/{name}.git"


def get_website_ppa():
    launchpad_username = get_github_username()
    return f"https://launchpanet/~{launchpad_username}/+archive/ubuntu/ppa"


def get_deps():
    try:
        mod = importlib.import_module("config.deps")
        if hasattr(mod, "packages"):
            return getattr(mod, "packages")
    except ModuleNotFoundError:
        pass
    return None



import glob
import os
import subprocess

import pydmt.helpers.general

protocol = "https"
codename = subprocess.check_output(["lsb_release", "--codename", "--short"]).decode().rstrip()
arch = subprocess.check_output(
    "dpkg-architecture | grep -e ^DEB_BUILD_ARCH= | cut -d = -f 2", shell=True
).decode().rstrip()
architectures = f"{arch} source"
component = "main"
folder = "apt"
service_dir = os.path.join(
    pydmt.helpers.general.homedir, "public_html/public", folder
)
pack_list = glob.glob(
    os.path.join(pydmt.helpers.general.homedir, "packages", "*.deb")
)
pack_str = " ".join(pack_list)
release_id = subprocess.check_output(["lsb_release", "--id", "--short"]).decode().rstrip()
key_file = "public_key.gpg"

personal_slug = 'veltzer'
exception = f'50{personal_slug}'
apache_site_file = f'{personal_slug}.apt'



"""
This module runs make
"""


from pydmt.utils.filesystem import mkdir_touch
from pydmt.utils.subprocess import check_call_ve

from pydmt.api.one_source_one_target import OneSourceOneTarget

from pydmt.configs import ConfigTarget

SOURCE_FILE = "Makefile"


class BuilderMake(OneSourceOneTarget):
    def build(self) -> None:
        args = [
            "make",
        ]
        if ConfigTarget.dev:
            args.append("DEV=0")
        check_call_ve(args)
        mkdir_touch(self.target)



from pydmt.api.feature import Feature
from pydmt.builders.apt import BuilderApt
from pydmt.core.pydmt import PyDMT
from pydmt.helpers.urls import get_deps


class FeatureApt(Feature):
    def __init__(
        self,
    ):
        self.packages = get_deps()

    def setup(self, pydmt: PyDMT) -> None:
        if self.packages is not None:
            pydmt.add_builder(BuilderApt(
                source="config/deps.py",
                target="out/deps.stamp",
                packages=self.packages
            ))



import setuptools


def get_readme():
    with open('README.rst') as f:
        return f.read()


setuptools.setup(
    # the first three fields are a must according to the documentation
    name="pydmt",
    version="0.1.93",
    packages=[
        "pydmt",
        "pydmt.api",
        "pydmt.builders",
        "pydmt.core",
        "pydmt.features",
        "pydmt.helpers",
        "pydmt.utils",
    ],
    # from here all is optional
    description="Python dependency management tool",
    long_description=get_readme(),
    long_description_content_type="text/x-rst",
    author="Mark Veltzer",
    author_email="mark.veltzer@gmail.com",
    maintainer="Mark Veltzer",
    maintainer_email="mark.veltzer@gmail.com",
    keywords=[
        "pydmt",
        "cons",
        "scons",
        "software construction",
        "make",
        "cmake",
        "maven",
        "mvn",
    ],
    url="https://veltzer.github.io/pydmt",
    download_url="https://github.com/veltzer/pydmt",
    license="MIT",
    platforms=[
        "python3",
    ],
    install_requires=[
        "pyfakeuse",
        "pylogconf",
        "pytconf",
        "mako",
        "sphinx",
        "pyyaml",
        "jsonschema",
        "venv-run",
        "gitpython",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={"console_scripts": [
        "pydmt=pydmt.main:main",
    ]},
)



from pydmt.api.one_source_one_target import OneSourceOneTarget


class Fail(OneSourceOneTarget):
    """
    This is a builder that fails.
    Why do we need it? Tests
    Why does it need a source and target?
    Because otherwise it would not get triggered
    """
    def build(self):
        raise ValueError("fail")
